@@ -1,7 +1,7 @@
 /*
  * xfrd.h - XFR (transfer) Daemon header file. Coordinates SOA updates.
  *
- * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -161,7 +161,7 @@ struct xfrd_zone {
 	uint8_t tcp_waiting;
 	/* next zone in waiting list */
 	xfrd_zone_t* tcp_waiting_next;
-	/* zone is waiting for a ucp connection (tcp is preferred) */
+	/* zone is waiting for a udp connection (tcp is preferred) */
 	uint8_t udp_waiting;
 	/* next zone in waiting list for UDP */
 	xfrd_zone_t* udp_waiting_next;
@@ -173,14 +173,13 @@ struct xfrd_zone {
 	uint32_t msg_old_serial, msg_new_serial; /* host byte order */
 	size_t msg_rr_count;
 	uint8_t msg_is_ixfr; /* 1:IXFR detected. 2:middle IXFR SOA seen. */
-#ifdef TSIG
 	tsig_record_type tsig; /* tsig state for IXFR/AXFR */
-#endif
 };
 
 enum xfrd_packet_result {
 	xfrd_packet_bad, /* drop the packet/connection */
 	xfrd_packet_more, /* more packets to follow on tcp */
+	xfrd_packet_notimpl, /* server responded with NOTIMPL or FORMATERR */
 	xfrd_packet_tcp, /* try tcp connection */
 	xfrd_packet_transfer, /* server responded with transfer*/
 	xfrd_packet_newlease /* no changes, soa OK */
@@ -234,7 +233,7 @@ void xfrd_make_request(xfrd_zone_t* zone);
  * send packet via udp (returns UDP fd source socket) to acl addr.
  * returns -1 on failure.
  */
-int xfrd_send_udp(acl_options_t* acl, buffer_type* packet);
+int xfrd_send_udp(acl_options_t* acl, buffer_type* packet, acl_options_t* ifc);
 
 /*
  * read from udp port packet into buffer, returns 0 on failure
@@ -269,6 +268,9 @@ void xfrd_handle_passed_packet(buffer_type* packet, int acl_num);
 /* send expiry notify for all zones to nsd (sets all dirty). */
 void xfrd_send_expy_all_zones();
 
+/* try to reopen the logfile. */
+void xfrd_reopen_logfile();
+
 /* copy SOA info from rr to soa struct. */
 void xfrd_copy_soa(xfrd_soa_t* soa, rr_type* rr);
 
@@ -279,7 +281,11 @@ void xfrd_check_failed_updates();
 /*
  * Prepare zones for a reload, this sets the times on the zones to be
  * before the current time, so the reload happens after.
-*/
+ */
 void xfrd_prepare_zones_for_reload();
 
+/* Bind a local interface to a socket descriptor, return 1 on success */
+int xfrd_bind_local_interface(int sockd, acl_options_t* ifc,
+	acl_options_t* acl, int tcp);
+
 #endif /* XFRD_H */
